package Test::DBIx::Class::Example::Schema::DefaultRS; {
	use base 'Test::DBIx::Class::Example::Schema::ResultSet';
} 1

__END__

=head1 NAME

Test::DBIx::Class::Example::Schema::DefaultRS - The default ResultSet

=head1 DESCRIPTION

L<DBIx::Class> will use a resultset class which shares a name with the Result
class.  If one does not exist, use this class as the default.

=head1 SEE ALSO

The following modules or resources may be of interest.

L<DBIx::Class::ResultSet>, L<Test::DBIx::Class::Schema::ResultSet>

=head1 AUTHOR

John Napiorkowski C<< <jjnapiork@cpan.org> >>

=head1 COPYRIGHT & LICENSE

Copyright 2009, John Napiorkowski C<< <jjnapiork@cpan.org> >>

This program is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

